@@ -742,8 +742,10 @@ static void enqueue(const int flags, const char *encoding, const char *sender,
 		skip_from_ = 0;
 	    }
 	    if (strip_cr == STRIP_CR_DO && type == REC_TYPE_NORM)
-		if (VSTRING_LEN(buf) > 0 && vstring_end(buf)[-1] == '\r')
+		if (VSTRING_LEN(buf) > 0 && vstring_end(buf)[-1] == '\r') {
 		    vstring_truncate(buf, VSTRING_LEN(buf) - 1);
+		    VSTRING_TERMINATE(buf);
+		}
 	    if ((flags & SM_FLAG_AEOF) && prev_type != REC_TYPE_CONT
 		&& VSTRING_LEN(buf) == 1 && *STR(buf) == '.')
 		break;
