@@ -252,20 +252,20 @@ int     smtp_helo(SMTP_STATE *state, NOCLOBBER int misc_flags)
 
 #endif
 
+    /*
+     * Prepare for disaster.
+     */
+    smtp_timeout_setup(state->session->stream, var_smtp_helo_tmout);
+    if ((except = vstream_setjmp(state->session->stream)) != 0)
+	return (smtp_stream_except(state, except,
+			      "performing the initial protocol handshake"));
+
     /*
      * If not recursing after STARTTLS, examine the server greeting banner
      * and decide if we are going to send EHLO as the next command.
      */
     if ((misc_flags & SMTP_MISC_FLAG_IN_STARTTLS) == 0) {
 
-	/*
-	 * Prepare for disaster.
-	 */
-	smtp_timeout_setup(state->session->stream, var_smtp_helo_tmout);
-	if ((except = vstream_setjmp(state->session->stream)) != 0)
-	    return (smtp_stream_except(state, except,
-				    "receiving the initial SMTP greeting"));
-
 	/*
 	 * Read and parse the server's SMTP greeting banner.
 	 */
@@ -338,81 +338,88 @@ int     smtp_helo(SMTP_STATE *state, NOCLOBBER int misc_flags)
 				   "host %s refused to talk to me: %s",
 				   session->namaddr,
 				   translit(resp->str, "\n", " ")));
-	return (0);
     }
 
     /*
-     * Determine what server EHLO keywords to ignore, typically to avoid
-     * inter-operability problems.
+     * No early returns allowed, to ensure consistent handling of TLS and
+     * SASL policies.
      */
-    if (smtp_ehlo_dis_maps == 0
-	|| (ehlo_words = maps_find(smtp_ehlo_dis_maps, state->session->addr, 0)) == 0)
-	ehlo_words = var_smtp_ehlo_dis_words;
-    discard_mask = ehlo_mask(ehlo_words);
-    if (discard_mask && !(discard_mask & EHLO_MASK_SILENT))
-	msg_info("discarding EHLO keywords: %s", str_ehlo_mask(discard_mask));
+    if (session->features & SMTP_FEATURE_ESMTP) {
 
-    /*
-     * Pick up some useful features offered by the SMTP server. XXX Until we
-     * have a portable routine to convert from string to off_t with proper
-     * overflow detection, ignore the message size limit advertised by the
-     * SMTP server. Otherwise, we might do the wrong thing when the server
-     * advertises a really huge message size limit.
-     * 
-     * XXX Allow for "code (SP|-) ehlo-keyword (SP|=) ehlo-param...", because
-     * MicroSoft implemented AUTH based on an old draft.
-     */
-    lines = resp->str;
-    for (n = 0; (words = mystrtok(&lines, "\n")) != 0; /* see below */ ) {
-	if (mystrtok(&words, "- ") && (word = mystrtok(&words, " \t=")) != 0) {
-	    if (n == 0) {
-		if (session->helo != 0)
-		    myfree(session->helo);
-		session->helo = lowercase(mystrdup(word));
-		if (strcasecmp(word, var_myhostname) == 0
-		    && (misc_flags & SMTP_MISC_FLAG_LOOP_DETECT) != 0) {
-		    msg_warn("host %s replied to HELO/EHLO with my own hostname %s",
-			     session->namaddr, var_myhostname);
-		    return (smtp_site_fail(state,
-		     (session->features & SMTP_FEATURE_BEST_MX) ? 550 : 450,
+	/*
+	 * Determine what server EHLO keywords to ignore, typically to avoid
+	 * inter-operability problems.
+	 */
+	if (smtp_ehlo_dis_maps == 0
+	    || (ehlo_words = maps_find(smtp_ehlo_dis_maps,
+				       state->session->addr, 0)) == 0)
+	    ehlo_words = var_smtp_ehlo_dis_words;
+	discard_mask = ehlo_mask(ehlo_words);
+	if (discard_mask && !(discard_mask & EHLO_MASK_SILENT))
+	    msg_info("discarding EHLO keywords: %s",
+		     str_ehlo_mask(discard_mask));
+
+	/*
+	 * Pick up some useful features offered by the SMTP server.
+	 * 
+	 * XXX Allow for "code (SP|-) ehlo-keyword (SP|=) ehlo-param...",
+	 * because MicroSoft implemented AUTH based on an old draft.
+	 */
+	lines = resp->str;
+	for (n = 0; (words = mystrtok(&lines, "\n")) != 0; /* see below */ ) {
+	    if (mystrtok(&words, "- ")
+		&& (word = mystrtok(&words, " \t=")) != 0) {
+		if (n == 0) {
+		    if (session->helo != 0)
+			myfree(session->helo);
+		    session->helo = lowercase(mystrdup(word));
+		    if (strcasecmp(word, var_myhostname) == 0
+			&& (misc_flags & SMTP_MISC_FLAG_LOOP_DETECT) != 0) {
+			msg_warn("host %s replied to HELO/EHLO"
+				 " with my own hostname %s",
+				 session->namaddr, var_myhostname);
+			return (smtp_site_fail(state,
+					       (session->features & SMTP_FEATURE_BEST_MX) ? 550 : 450,
 					 "mail for %s loops back to myself",
-					   request->nexthop));
-		}
-	    } else if (strcasecmp(word, "8BITMIME") == 0) {
-		if ((discard_mask & EHLO_MASK_8BITMIME) == 0)
-		    session->features |= SMTP_FEATURE_8BITMIME;
-	    } else if (strcasecmp(word, "PIPELINING") == 0) {
-		if ((discard_mask & EHLO_MASK_PIPELINING) == 0)
-		    session->features |= SMTP_FEATURE_PIPELINING;
-	    } else if (strcasecmp(word, "XFORWARD") == 0) {
-		if ((discard_mask & EHLO_MASK_XFORWARD) == 0)
-		    while ((word = mystrtok(&words, " \t")) != 0)
-			session->features |= name_code(xforward_features,
+					       request->nexthop));
+		    }
+		} else if (strcasecmp(word, "8BITMIME") == 0) {
+		    if ((discard_mask & EHLO_MASK_8BITMIME) == 0)
+			session->features |= SMTP_FEATURE_8BITMIME;
+		} else if (strcasecmp(word, "PIPELINING") == 0) {
+		    if ((discard_mask & EHLO_MASK_PIPELINING) == 0)
+			session->features |= SMTP_FEATURE_PIPELINING;
+		} else if (strcasecmp(word, "XFORWARD") == 0) {
+		    if ((discard_mask & EHLO_MASK_XFORWARD) == 0)
+			while ((word = mystrtok(&words, " \t")) != 0)
+			    session->features |= name_code(xforward_features,
 						 NAME_CODE_FLAG_NONE, word);
-	    } else if (strcasecmp(word, "SIZE") == 0) {
-		if ((discard_mask & EHLO_MASK_SIZE) == 0) {
-		    session->features |= SMTP_FEATURE_SIZE;
-		    if ((word = mystrtok(&words, " \t")) != 0) {
-			if (!alldig(word))
-			    msg_warn("bad EHLO SIZE limit \"%s\" from %s",
-				     word, session->namaddr);
-			else
-			    session->size_limit = off_cvt_string(word);
+		} else if (strcasecmp(word, "SIZE") == 0) {
+		    if ((discard_mask & EHLO_MASK_SIZE) == 0) {
+			session->features |= SMTP_FEATURE_SIZE;
+			if ((word = mystrtok(&words, " \t")) != 0) {
+			    if (!alldig(word))
+				msg_warn("bad EHLO SIZE limit \"%s\" from %s",
+					 word, session->namaddr);
+			    else
+				session->size_limit = off_cvt_string(word);
+			}
 		    }
-		}
 #ifdef USE_TLS
-	    } else if (strcasecmp(word, "STARTTLS") == 0) {
-		/* Ignored later if we already sent STARTTLS. */
-		if ((discard_mask & EHLO_MASK_STARTTLS) == 0)
-		    session->features |= SMTP_FEATURE_STARTTLS;
+		} else if (strcasecmp(word, "STARTTLS") == 0) {
+		    /* Ignored later if we already sent STARTTLS. */
+		    if ((discard_mask & EHLO_MASK_STARTTLS) == 0)
+			session->features |= SMTP_FEATURE_STARTTLS;
 #endif
 #ifdef USE_SASL_AUTH
-	    } else if (var_smtp_sasl_enable && strcasecmp(word, "AUTH") == 0) {
-		if ((discard_mask & EHLO_MASK_AUTH) == 0)
-		    smtp_sasl_helo_auth(session, words);
+		} else if (var_smtp_sasl_enable
+			   && strcasecmp(word, "AUTH") == 0) {
+		    if ((discard_mask & EHLO_MASK_AUTH) == 0)
+			smtp_sasl_helo_auth(session, words);
 #endif
+		}
+		n++;
 	    }
-	    n++;
 	}
     }
     if (msg_verbose)
